@@ -1,4 +1,4 @@
-use cargo::util::{CliResult, CliError, Config};
+use cargo::util::{CliResult, CliError, Config, human};
 
 #[derive(RustcDecodable)]
 pub struct Options;
@@ -14,9 +14,9 @@ Options:
     -h, --help          Print this message
 ";
 
-pub fn execute(_: Options, _: &Config) -> CliResult<Option<()>> {
+pub fn execute(_: Options, _: &Config) -> CliResult {
     // This is a dummy command just so that `cargo help help` works.
     // The actual delegation of help flag to subcommands is handled by the
     // cargo command.
-    Err(CliError::new("Help command should not be executed directly.", 101))
+    Err(CliError::new(human("help command should not be executed directly"), 101))
 }
