@@ -59,7 +59,7 @@ pub struct ManifestMetadata {
     pub badges: HashMap<String, HashMap<String, String>>,
 }
 
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
 pub enum LibKind {
     Lib,
     Rlib,
@@ -101,7 +101,7 @@ impl LibKind {
     }
 }
 
-#[derive(Debug, Clone, Hash, PartialEq, Eq)]
+#[derive(Debug, Clone, Hash, PartialEq, Eq, PartialOrd, Ord)]
 pub enum TargetKind {
     Lib(Vec<LibKind>),
     Bin,
@@ -112,23 +112,6 @@ pub enum TargetKind {
     CustomBuild,
 }
 
-impl TargetKind {
-    /// Returns a vector of crate types as specified in a manifest
-    pub fn crate_types(&self) -> Vec<&str> {
-        use self::TargetKind::*;
-        match *self {
-            Lib(ref kinds) | ExampleLib(ref kinds) => {
-                kinds.iter().map(LibKind::crate_type).collect()
-            }
-            Bin => vec!["bin"],
-            ExampleBin => vec!["example"],
-            Test => vec!["test"],
-            CustomBuild => vec!["custom-build"],
-            Bench => vec!["bench"]
-        }
-    }
-}
-
 impl ser::Serialize for TargetKind {
     fn serialize<S>(&self, s: S) -> Result<S::Ok, S::Error>
         where S: ser::Serializer,
@@ -162,6 +145,7 @@ pub struct Profile {
     pub rustdoc_args: Option<Vec<String>>,
     pub debuginfo: Option<u32>,
     pub debug_assertions: bool,
+    pub overflow_checks: bool,
     #[serde(skip_serializing)]
     pub rpath: bool,
     pub test: bool,
@@ -207,7 +191,11 @@ pub struct Target {
 
 #[derive(Serialize)]
 struct SerializedTarget<'a> {
+    /// Is this a `--bin bin`, `--lib`, `--example ex`?
+    /// Serialized as a list of strings for historical reasons.
     kind: &'a TargetKind,
+    /// Corresponds to `--crate-type` compiler attribute.
+    /// See https://doc.rust-lang.org/reference.html#linkage
     crate_types: Vec<&'a str>,
     name: &'a str,
     src_path: &'a PathBuf,
@@ -217,7 +205,7 @@ impl ser::Serialize for Target {
     fn serialize<S: ser::Serializer>(&self, s: S) -> Result<S::Ok, S::Error> {
         SerializedTarget {
             kind: &self.kind,
-            crate_types: self.kind.crate_types(),
+            crate_types: self.rustc_crate_types(),
             name: &self.name,
             src_path: &self.src_path,
         }.serialize(s)
@@ -541,6 +529,7 @@ impl Profile {
         Profile {
             debuginfo: Some(2),
             debug_assertions: true,
+            overflow_checks: true,
             ..Profile::default()
         }
     }
@@ -607,6 +596,7 @@ impl Default for Profile {
             rustdoc_args: None,
             debuginfo: None,
             debug_assertions: false,
+            overflow_checks: false,
             rpath: false,
             test: false,
             doc: false,
