@@ -10,13 +10,13 @@ use git2::Config as GitConfig;
 use term::color::BLACK;
 
 use core::Workspace;
-use util::{GitRepo, HgRepo, CargoResult, human, ChainError, internal};
+use util::{GitRepo, HgRepo, PijulRepo, CargoResult, human, ChainError, internal};
 use util::{Config, paths};
 
 use toml;
 
 #[derive(Clone, Copy, Debug, PartialEq)]
-pub enum VersionControl { Git, Hg, NoVcs }
+pub enum VersionControl { Git, Hg, Pijul, NoVcs }
 
 pub struct NewOptions<'a> {
     pub version_control: Option<VersionControl>,
@@ -45,6 +45,7 @@ impl Decodable for VersionControl {
         Ok(match &d.read_str()?[..] {
             "git" => VersionControl::Git,
             "hg" => VersionControl::Hg,
+            "pijul" => VersionControl::Pijul,
             "none" => VersionControl::NoVcs,
             n => {
                 let err = format!("could not decode '{}' as version control", n);
@@ -331,10 +332,15 @@ pub fn init(opts: NewOptions, config: &Config) -> CargoResult<()> {
             num_detected_vsces += 1;
         }
 
+        if fs::metadata(&path.join(".pijul")).is_ok() {
+            version_control = Some(VersionControl::Pijul);
+            num_detected_vsces += 1;
+        }
+
         // if none exists, maybe create git, like in `cargo new`
 
         if num_detected_vsces > 1 {
-            bail!("both .git and .hg directories found \
+            bail!("more than one of .hg, .git, or .pijul directories found \
                               and the ignore file can't be \
                               filled in as a result, \
                               specify --vcs to override detection");
@@ -401,13 +407,18 @@ fn mk(config: &Config, opts: &MkOptions) -> CargoResult<()> {
             }
             paths::append(&path.join(".hgignore"), ignore.as_bytes())?;
         },
+        VersionControl::Pijul => {
+            if !fs::metadata(&path.join(".pijul")).is_ok() {
+                PijulRepo::init(path, config.cwd())?;
+            }
+        },
         VersionControl::NoVcs => {
             fs::create_dir_all(path)?;
         },
     };
 
     let (author_name, email) = discover_author()?;
-    // Hoo boy, sure glad we've got exhaustivenes checking behind us.
+    // Hoo boy, sure glad we've got exhaustiveness checking behind us.
     let author = match (cfg.name, cfg.email, author_name, email) {
         (Some(name), Some(email), _, _) |
         (Some(name), None, _, Some(email)) |
@@ -419,7 +430,7 @@ fn mk(config: &Config, opts: &MkOptions) -> CargoResult<()> {
 
     let mut cargotoml_path_specifier = String::new();
 
-    // Calculare what [lib] and [[bin]]s do we need to append to Cargo.toml
+    // Calculate what [lib] and [[bin]]s do we need to append to Cargo.toml
 
     for i in &opts.source_files {
         if i.bin {
