@@ -1,24 +1,36 @@
 use std::path::Path;
 
-use ops::{self, CompileFilter};
-use util::{self, CargoResult, process, ProcessError};
-use core::Package;
+use ops::{self, CompileFilter, Packages};
+use util::{self, human, CargoResult, ProcessError};
+use core::Workspace;
 
-pub fn run(manifest_path: &Path,
+pub fn run(ws: &Workspace,
            options: &ops::CompileOptions,
            args: &[String]) -> CargoResult<Option<ProcessError>> {
-    let config = options.config;
-    let root = try!(Package::for_path(manifest_path, config));
+    let config = ws.config();
 
-    let mut bins = root.manifest().targets().iter().filter(|a| {
+    let pkg = match options.spec {
+        Packages::All => unreachable!("cargo run supports single package only"),
+        Packages::Packages(xs) => match xs.len() {
+            0 => ws.current()?,
+            1 => ws.members()
+                .find(|pkg| pkg.name() == xs[0])
+                .ok_or_else(|| human(
+                    format!("package `{}` is not a member of the workspace", xs[0])
+                ))?,
+            _ => unreachable!("cargo run supports single package only"),
+        }
+    };
+
+    let mut bins = pkg.manifest().targets().iter().filter(|a| {
         !a.is_lib() && !a.is_custom_build() && match options.filter {
-            CompileFilter::Everything => a.is_bin(),
+            CompileFilter::Everything { .. } => a.is_bin(),
             CompileFilter::Only { .. } => options.filter.matches(a),
         }
     });
     if bins.next().is_none() {
         match options.filter {
-            CompileFilter::Everything => {
+            CompileFilter::Everything { .. } => {
                 bail!("a bin target must be available for `cargo run`")
             }
             CompileFilter::Only { .. } => {
@@ -28,7 +40,7 @@ pub fn run(manifest_path: &Path,
     }
     if bins.next().is_some() {
         match options.filter {
-            CompileFilter::Everything => {
+            CompileFilter::Everything { .. } => {
                 bail!("`cargo run` requires that a project only have one \
                        executable; use the `--bin` option to specify which one \
                        to run")
@@ -40,7 +52,8 @@ pub fn run(manifest_path: &Path,
         }
     }
 
-    let compile = try!(ops::compile(manifest_path, options));
+    let compile = ops::compile(ws, options)?;
+    assert_eq!(compile.binaries.len(), 1);
     let exe = &compile.binaries[0];
     let exe = match util::without_prefix(&exe, config.cwd()) {
         Some(path) if path.file_name() == Some(path.as_os_str())
@@ -48,10 +61,9 @@ pub fn run(manifest_path: &Path,
         Some(path) => path.to_path_buf(),
         None => exe.to_path_buf(),
     };
-    let mut process = try!(compile.target_process(exe, &root))
-                                  .into_process_builder();
+    let mut process = compile.target_process(exe, &pkg)?;
     process.args(args).cwd(config.cwd());
 
-    try!(config.shell().status("Running", process.to_string()));
-    Ok(process.exec().err())
+    config.shell().status("Running", process.to_string())?;
+    Ok(process.exec_replace().err())
 }
