@@ -147,6 +147,7 @@ pub fn compile_targets<'a, 'cfg: 'a>(ws: &Workspace<'cfg>,
 
             if unit.profile.test {
                 cx.compilation.tests.push((unit.pkg.clone(),
+                                           unit.target.kind().clone(),
                                            unit.target.name().to_string(),
                                            dst));
             } else if unit.target.is_bin() || unit.target.is_example() {
@@ -249,12 +250,16 @@ fn rustc(cx: &mut Context, unit: &Unit, exec: Arc<Executor>) -> CargoResult<Work
     let mut rustc = prepare_rustc(cx, crate_types, unit)?;
 
     let name = unit.pkg.name().to_string();
+
+    // If this is an upstream dep we don't want warnings from, turn off all
+    // lints.
     if !cx.show_warnings(unit.pkg.package_id()) {
-        if cx.config.rustc()?.cap_lints {
-            rustc.arg("--cap-lints").arg("allow");
-        } else {
-            rustc.arg("-Awarnings");
-        }
+        rustc.arg("--cap-lints").arg("allow");
+
+    // If this is an upstream dep but we *do* want warnings, make sure that they
+    // don't fail compilation.
+    } else if !unit.pkg.package_id().source_id().is_path() {
+        rustc.arg("--cap-lints").arg("warn");
     }
 
     let filenames = cx.target_filenames(unit)?;
@@ -592,8 +597,8 @@ fn build_base_args(cx: &mut Context,
                    crate_types: &[&str]) {
     let Profile {
         ref opt_level, lto, codegen_units, ref rustc_args, debuginfo,
-        debug_assertions, rpath, test, doc: _doc, run_custom_build,
-        ref panic, rustdoc_args: _, check,
+        debug_assertions, overflow_checks, rpath, test, doc: _doc,
+        run_custom_build, ref panic, rustdoc_args: _, check,
     } = *unit.profile;
     assert!(!run_custom_build);
 
@@ -673,10 +678,27 @@ fn build_base_args(cx: &mut Context,
         cmd.args(args);
     }
 
-    if debug_assertions && opt_level != "0" {
-        cmd.args(&["-C", "debug-assertions=on"]);
-    } else if !debug_assertions && opt_level == "0" {
-        cmd.args(&["-C", "debug-assertions=off"]);
+    // -C overflow-checks is implied by the setting of -C debug-assertions,
+    // so we only need to provide -C overflow-checks if it differs from
+    // the value of -C debug-assertions we would provide.
+    if opt_level != "0" {
+        if debug_assertions {
+            cmd.args(&["-C", "debug-assertions=on"]);
+            if !overflow_checks {
+                cmd.args(&["-C", "overflow-checks=off"]);
+            }
+        } else if overflow_checks {
+            cmd.args(&["-C", "overflow-checks=on"]);
+        }
+    } else {
+        if !debug_assertions {
+            cmd.args(&["-C", "debug-assertions=off"]);
+            if overflow_checks {
+                cmd.args(&["-C", "overflow-checks=on"]);
+            }
+        } else if !overflow_checks {
+            cmd.args(&["-C", "overflow-checks=off"]);
+        }
     }
 
     if test && unit.target.harness() {
