@@ -17,6 +17,7 @@ use core::shell::{Verbosity, ColorConfig};
 use core::MultiShell;
 use util::{CargoResult, CargoError, ChainError, Rustc, internal, human};
 use util::{Filesystem, LazyCell};
+use util::paths;
 
 use util::toml as cargo_toml;
 
@@ -92,7 +93,8 @@ impl Config {
     }
 
     pub fn rustc(&self) -> CargoResult<&Rustc> {
-        self.rustc.get_or_try_init(|| Rustc::new(self.get_tool("rustc")?))
+        self.rustc.get_or_try_init(|| Rustc::new(self.get_tool("rustc")?,
+                                                 self.maybe_get_tool("rustc_wrapper")?))
     }
 
     pub fn cargo_exe(&self) -> CargoResult<&Path> {
@@ -108,6 +110,16 @@ impl Config {
         self.values.get_or_try_init(|| self.load_values())
     }
 
+    pub fn set_values(&self, values: HashMap<String, ConfigValue>) -> CargoResult<()> {
+        if self.values.borrow().is_some() {
+            return Err(human("Config values already found"));
+        }
+        match self.values.fill(values) {
+            Ok(()) => Ok(()),
+            Err(_) => Err(human("Could not fill values")),
+        }
+    }
+
     pub fn cwd(&self) -> &Path { &self.cwd }
 
     pub fn target_dir(&self) -> CargoResult<Option<Filesystem>> {
@@ -377,7 +389,7 @@ impl Config {
         !self.frozen.get() && !self.locked.get()
     }
 
-    fn load_values(&self) -> CargoResult<HashMap<String, ConfigValue>> {
+    pub fn load_values(&self) -> CargoResult<HashMap<String, ConfigValue>> {
         let mut cfg = CV::Table(HashMap::new(), PathBuf::from("."));
 
         walk_tree(&self.cwd, |mut file, path| {
@@ -404,18 +416,27 @@ impl Config {
         }
     }
 
-    fn get_tool(&self, tool: &str) -> CargoResult<PathBuf> {
+    /// Look for a path for `tool` in an environment variable or config path, but return `None`
+    /// if it's not present.
+    fn maybe_get_tool(&self, tool: &str) -> CargoResult<Option<PathBuf>> {
         let var = tool.chars().flat_map(|c| c.to_uppercase()).collect::<String>();
         if let Some(tool_path) = env::var_os(&var) {
-            return Ok(PathBuf::from(tool_path));
+            return Ok(Some(PathBuf::from(tool_path)));
         }
 
         let var = format!("build.{}", tool);
         if let Some(tool_path) = self.get_path(&var)? {
-            return Ok(tool_path.val);
+            return Ok(Some(tool_path.val));
         }
 
-        Ok(PathBuf::from(tool))
+        Ok(None)
+    }
+
+    /// Look for a path for `tool` in an environment variable or config path, defaulting to `tool`
+    /// as a path.
+    fn get_tool(&self, tool: &str) -> CargoResult<PathBuf> {
+        self.maybe_get_tool(tool)
+            .map(|t| t.unwrap_or(PathBuf::from(tool)))
     }
 }
 
@@ -698,10 +719,9 @@ pub fn homedir(cwd: &Path) -> Option<PathBuf> {
 fn walk_tree<F>(pwd: &Path, mut walk: F) -> CargoResult<()>
     where F: FnMut(File, &Path) -> CargoResult<()>
 {
-    let mut current = pwd;
     let mut stash: HashSet<PathBuf> = HashSet::new();
 
-    loop {
+    for current in paths::ancestors(pwd) {
         let possible = current.join(".cargo").join("config");
         if fs::metadata(&possible).is_ok() {
             let file = File::open(&possible)?;
@@ -710,11 +730,6 @@ fn walk_tree<F>(pwd: &Path, mut walk: F) -> CargoResult<()>
 
             stash.insert(possible);
         }
-
-        match current.parent() {
-            Some(p) => current = p,
-            None => break,
-        }
     }
 
     // Once we're done, also be sure to walk the home directory even if it's not
